@@ -23,17 +23,18 @@ build_time_vars = {'ABIFLAGS': '',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc',
  'CCSHARED': '',
- 'CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 -I/usr/include '
-           '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 -I/usr/include '
-           '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 -I/usr/include '
-           '-I/usr/pkg/include -DNCURSES_WIDECHAR=1',
+ 'CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 -pipe '
+           '-I/usr/include -DOPENSSL_THREADS=1 -I/usr/pkg/include '
+           '-DNCURSES_WIDECHAR=1 -O2 -pipe -I/usr/include -DOPENSSL_THREADS=1 '
+           '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 -pipe -I/usr/include '
+           '-DOPENSSL_THREADS=1 -I/usr/pkg/include -DNCURSES_WIDECHAR=1',
  'CFLAGSFORSHARED': '',
  'CFLAGS_ALIASING': '',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
- 'CONFIGURE_CFLAGS': '-O2 -I/usr/include -I/usr/pkg/include '
-                     '-DNCURSES_WIDECHAR=1',
+ 'CONFIGURE_CFLAGS': '-O2 -pipe -I/usr/include -DOPENSSL_THREADS=1 '
+                     '-I/usr/pkg/include -DNCURSES_WIDECHAR=1',
  'CONFIGURE_CFLAGS_NODIST': '-std=gnu99 -Wextra -Wno-unused-parameter '
                             '-Wno-missing-field-initializers '
                             '-fvisibility=hidden',
@@ -47,21 +48,20 @@ build_time_vars = {'ABIFLAGS': '',
                 "'--without-pymalloc' '--disable-rpath' '--prefix=/usr/pkg' "
                 "'--build=powerpc-apple-darwin8' "
                 "'--host=powerpc-apple-darwin8' '--mandir=/usr/pkg/man' "
-                "'--enable-option-checking=yes' "
                 "'build_alias=powerpc-apple-darwin8' "
-                "'host_alias=powerpc-apple-darwin8' 'CC=cc' 'CFLAGS=-O2 "
+                "'host_alias=powerpc-apple-darwin8' 'CC=cc' 'CFLAGS=-O2 -pipe "
+                '-I/usr/include -DOPENSSL_THREADS=1 -I/usr/pkg/include '
+                "-DNCURSES_WIDECHAR=1' 'LDFLAGS=-Wl,-m -L/usr/lib "
+                "-L/usr/pkg/lib' 'LIBS=' 'CPPFLAGS=-isystem /usr/include "
                 "-I/usr/include -I/usr/pkg/include -DNCURSES_WIDECHAR=1' "
-                "'LDFLAGS=-Wl,-m -L/usr/lib -L/usr/pkg/lib' 'LIBS=' "
-                "'CPPFLAGS=-isystem /usr/include -I/usr/include "
-                "-I/usr/pkg/include -DNCURSES_WIDECHAR=1' "
-                "'PKG_CONFIG=/Users/nia/pkgsrc/lang/python310/work/.tools/bin/pkg-config' "
+                "'PKG_CONFIG=/Users/nia/dreckly/lang/python310/work/.tools/bin/pkg-config' "
                 "'PKG_CONFIG_PATH=' "
-                "'PKG_CONFIG_LIBDIR=/Users/nia/pkgsrc/lang/python310/work/.buildlink/lib/pkgconfig:/Users/nia/pkgsrc/lang/python310/work/.buildlink/share/pkgconfig'",
+                "'PKG_CONFIG_LIBDIR=/Users/nia/dreckly/lang/python310/work/.buildlink/lib/pkgconfig:/Users/nia/dreckly/lang/python310/work/.buildlink/share/pkgconfig'",
  'CONFINCLUDEDIR': '/usr/pkg/include',
  'CONFINCLUDEPY': '/usr/pkg/include/python3.10',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15/coverage.info',
- 'COVERAGE_REPORT': '/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15/lcov-report',
+ 'COVERAGE_INFO': '/Users/nia/dreckly/lang/python310/work/Python-3.10.18/coverage.info',
+ 'COVERAGE_REPORT': '/Users/nia/dreckly/lang/python310/work/Python-3.10.18/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -isystem /usr/include -I/usr/include '
@@ -545,11 +545,11 @@ build_time_vars = {'ABIFLAGS': '',
                    '/usr/pkg/include/python3.10 /usr/pkg/include/python3.10',
  'INCLUDEDIR': '/usr/pkg/include',
  'INCLUDEPY': '/usr/pkg/include/python3.10',
- 'INSTALL': '/usr/bin/install -c -o root -g wheel',
- 'INSTALL_DATA': '/usr/bin/install -c -o root -g wheel -m 644',
- 'INSTALL_PROGRAM': '/usr/bin/install -c  -o root -g wheel -m 755',
- 'INSTALL_SCRIPT': '/usr/bin/install -c -o root -g wheel -m 755',
- 'INSTALL_SHARED': '/usr/bin/install -c -o root -g wheel -m 755',
+ 'INSTALL': '/usr/bin/install -c -o nia -g nia',
+ 'INSTALL_DATA': '/usr/bin/install -c -o nia -g nia -m 644',
+ 'INSTALL_PROGRAM': '/usr/bin/install -c  -o nia -g nia -m 755',
+ 'INSTALL_SCRIPT': '/usr/bin/install -c -o nia -g nia -m 755',
+ 'INSTALL_SHARED': '/usr/bin/install -c -o nia -g nia -m 755',
  'INSTSONAME': 'libpython3.10.dylib',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -625,7 +625,8 @@ build_time_vars = {'ABIFLAGS': '',
  'OPENSSL_LDFLAGS': '-L/usr/pkg/lib',
  'OPENSSL_LIBS': '-lssl -lcrypto',
  'OPENSSL_RPATH': '',
- 'OPT': '-DNDEBUG -O2 -I/usr/include -I/usr/pkg/include -DNCURSES_WIDECHAR=1',
+ 'OPT': '-DNDEBUG -O2 -pipe -I/usr/include -DOPENSSL_THREADS=1 '
+        '-I/usr/pkg/include -DNCURSES_WIDECHAR=1',
  'OTHER_LIBTOOL_OPT': '',
  'PACKAGE_BUGREPORT': 0,
  'PACKAGE_NAME': 0,
@@ -660,36 +661,41 @@ build_time_vars = {'ABIFLAGS': '',
  'PYTHON_OBJS': '\\',
  'PY_BUILTIN_HASHLIB_HASHES': '"md5,sha1,sha256,sha512,sha3,blake2"',
  'PY_BUILTIN_MODULE_CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 '
-                             '-I/usr/include -I/usr/pkg/include '
-                             '-DNCURSES_WIDECHAR=1 -O2 -I/usr/include '
+                             '-pipe -I/usr/include -DOPENSSL_THREADS=1 '
                              '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 '
-                             '-I/usr/include -I/usr/pkg/include '
-                             '-DNCURSES_WIDECHAR=1 -std=gnu99 -Wextra '
-                             '-Wno-unused-parameter '
+                             '-pipe -I/usr/include -DOPENSSL_THREADS=1 '
+                             '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 '
+                             '-pipe -I/usr/include -DOPENSSL_THREADS=1 '
+                             '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 '
+                             '-std=gnu99 -Wextra -Wno-unused-parameter '
                              '-Wno-missing-field-initializers '
                              '-fvisibility=hidden  -I./Include/internal -I. '
                              '-I./Include -isystem /usr/include -I/usr/include '
                              '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -isystem '
                              '/usr/include -I/usr/include -I/usr/pkg/include '
                              '-DNCURSES_WIDECHAR=1 -DPy_BUILD_CORE_BUILTIN',
- 'PY_CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 -I/usr/include '
-              '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 -I/usr/include '
-              '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 -I/usr/include '
-              '-I/usr/pkg/include -DNCURSES_WIDECHAR=1',
+ 'PY_CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 -pipe '
+              '-I/usr/include -DOPENSSL_THREADS=1 -I/usr/pkg/include '
+              '-DNCURSES_WIDECHAR=1 -O2 -pipe -I/usr/include '
+              '-DOPENSSL_THREADS=1 -I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 '
+              '-pipe -I/usr/include -DOPENSSL_THREADS=1 -I/usr/pkg/include '
+              '-DNCURSES_WIDECHAR=1',
  'PY_CFLAGS_NODIST': '-std=gnu99 -Wextra -Wno-unused-parameter '
                      '-Wno-missing-field-initializers -fvisibility=hidden  '
                      '-I./Include/internal',
  'PY_COERCE_C_LOCALE': 1,
- 'PY_CORE_CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 '
-                   '-I/usr/include -I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 '
-                   '-I/usr/include -I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 '
-                   '-I/usr/include -I/usr/pkg/include -DNCURSES_WIDECHAR=1 '
-                   '-std=gnu99 -Wextra -Wno-unused-parameter '
-                   '-Wno-missing-field-initializers -fvisibility=hidden  '
-                   '-I./Include/internal -I. -I./Include -isystem /usr/include '
-                   '-I/usr/include -I/usr/pkg/include -DNCURSES_WIDECHAR=1 '
+ 'PY_CORE_CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 -pipe '
+                   '-I/usr/include -DOPENSSL_THREADS=1 -I/usr/pkg/include '
+                   '-DNCURSES_WIDECHAR=1 -O2 -pipe -I/usr/include '
+                   '-DOPENSSL_THREADS=1 -I/usr/pkg/include '
+                   '-DNCURSES_WIDECHAR=1 -O2 -pipe -I/usr/include '
+                   '-DOPENSSL_THREADS=1 -I/usr/pkg/include '
+                   '-DNCURSES_WIDECHAR=1 -std=gnu99 -Wextra '
+                   '-Wno-unused-parameter -Wno-missing-field-initializers '
+                   '-fvisibility=hidden  -I./Include/internal -I. -I./Include '
                    '-isystem /usr/include -I/usr/include -I/usr/pkg/include '
-                   '-DNCURSES_WIDECHAR=1 -DPy_BUILD_CORE',
+                   '-DNCURSES_WIDECHAR=1 -isystem /usr/include -I/usr/include '
+                   '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -DPy_BUILD_CORE',
  'PY_CORE_LDFLAGS': '-L. -Wl,-m -L/usr/lib -L/usr/pkg/lib -Wl,-m -L/usr/lib '
                     '-L/usr/pkg/lib',
  'PY_CPPFLAGS': '-I. -I./Include -isystem /usr/include -I/usr/include '
@@ -703,10 +709,11 @@ build_time_vars = {'ABIFLAGS': '',
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
  'PY_STDMODULE_CFLAGS': '-fno-strict-aliasing -Wsign-compare -DNDEBUG -O2 '
-                        '-I/usr/include -I/usr/pkg/include '
-                        '-DNCURSES_WIDECHAR=1 -O2 -I/usr/include '
-                        '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 '
-                        '-I/usr/include -I/usr/pkg/include '
+                        '-pipe -I/usr/include -DOPENSSL_THREADS=1 '
+                        '-I/usr/pkg/include -DNCURSES_WIDECHAR=1 -O2 -pipe '
+                        '-I/usr/include -DOPENSSL_THREADS=1 -I/usr/pkg/include '
+                        '-DNCURSES_WIDECHAR=1 -O2 -pipe -I/usr/include '
+                        '-DOPENSSL_THREADS=1 -I/usr/pkg/include '
                         '-DNCURSES_WIDECHAR=1 -std=gnu99 -Wextra '
                         '-Wno-unused-parameter -Wno-missing-field-initializers '
                         '-fvisibility=hidden  -I./Include/internal -I. '
@@ -722,7 +729,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': ':',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'DYLD_LIBRARY_PATH=/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15',
+ 'RUNSHARED': 'DYLD_LIBRARY_PATH=/Users/nia/dreckly/lang/python310/work/Python-3.10.18:/usr/pkg/lib',
  'SCRIPTDIR': '/usr/pkg/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -764,10 +771,10 @@ build_time_vars = {'ABIFLAGS': '',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'DYLD_LIBRARY_PATH=/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15 '
+ 'TESTPYTHON': 'DYLD_LIBRARY_PATH=/Users/nia/dreckly/lang/python310/work/Python-3.10.18:/usr/pkg/lib '
                './python.exe',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'DYLD_LIBRARY_PATH=/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15 '
+ 'TESTRUNNER': 'DYLD_LIBRARY_PATH=/Users/nia/dreckly/lang/python310/work/Python-3.10.18:/usr/pkg/lib '
                './python.exe ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -795,8 +802,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15',
- 'abs_srcdir': '/Users/nia/pkgsrc/lang/python310/work/Python-3.10.15',
+ 'abs_builddir': '/Users/nia/dreckly/lang/python310/work/Python-3.10.18',
+ 'abs_srcdir': '/Users/nia/dreckly/lang/python310/work/Python-3.10.18',
  'datarootdir': '/usr/pkg/share',
  'exec_prefix': '/usr/pkg',
  'prefix': '/usr/pkg',
