@@ -28,7 +28,7 @@ DEFAULT_FRAMEWORK_FALLBACK = [
 
 DEFAULT_LIBRARY_FALLBACK = [
     os.path.expanduser("~/lib"),
-    "/2022Q3/lib",
+    "/usr/pkg/lib",
     "/lib",
     "/usr/lib",
 ]
