@@ -4,10 +4,6 @@ Utility functions for simple, timestamp-based dependency of files
 and groups of files; also, function based entirely on such
 timestamp dependency analysis."""
 
-# This module should be kept compatible with Python 1.5.2.
-
-__revision__ = "$Id: dep_util.py 29762 2002-11-19 13:12:28Z akuchling $"
-
 import os
 from distutils.errors import DistutilsFileError
 
@@ -19,7 +15,8 @@ def newer (source, target):
     Raise DistutilsFileError if 'source' does not exist.
     """
     if not os.path.exists(source):
-        raise DistutilsFileError, "file '%s' does not exist" % source
+        raise DistutilsFileError("file '%s' does not exist" %
+                                 os.path.abspath(source))
     if not os.path.exists(target):
         return 1
 
@@ -39,7 +36,7 @@ def newer_pairwise (sources, targets):
     of 'newer()'.
     """
     if len(sources) != len(targets):
-        raise ValueError, "'sources' and 'targets' must be same length"
+        raise ValueError("'sources' and 'targets' must be same length")
 
     # build a pair of lists (sources, targets) where  source is newer
     n_sources = []
