@@ -1,4 +1,4 @@
-#!/2022Q3/bin/python3.10
+#!/usr/pkg/bin/python3.10
 
 """The Tab Nanny despises ambiguous indentation.  She knows no mercy.
 
@@ -23,8 +23,6 @@ __version__ = "6"
 import os
 import sys
 import tokenize
-if not hasattr(tokenize, 'NL'):
-    raise ValueError("tokenize.NL doesn't exist -- tokenize module too old")
 
 __all__ = ["check", "NannyNag", "process_tokens"]
 
