@@ -6,6 +6,7 @@ import signal
 import socket
 import sys
 from test import support
+from test.support import os_helper
 from test.support import socket_helper
 from time import sleep
 import unittest
@@ -539,7 +540,7 @@ class KqueueSelectorTestCase(BaseSelectorTestCase, ScalableSelectorMixIn,
         # a file descriptor that's been closed should raise an OSError
         # with EBADF
         s = self.SELECTOR()
-        bad_f = support.make_bad_fd()
+        bad_f = os_helper.make_bad_fd()
         with self.assertRaises(OSError) as cm:
             s.register(bad_f, selectors.EVENT_READ)
         self.assertEqual(cm.exception.errno, errno.EBADF)
